
#include <avr/io.h>
#include <avr/interrupt.h>
#include <modem.h>

/******************************************************************************
	Function : Init_Modem
	Description : Initialize the USART and Check If Modem is Connected
	Inputs : none
	Outputs : none
******************************************************************************/
void
Init_Modem()
{
	/***** Initialise USART *****/
	//Set Baud Rate as per user
	UBRRH = USART_Baud_Higher_Byte;
	UBRRL = USART_Baud_Lower_Byte;
	//Enabel Receiver & Transmitter
	UCSRB = (1<<RXEN)|(1<<TXEN);
	//Set Frame Format Asynchronous 8-bits & 1 Stop Bits
	UCSRC = 0x86;
	return;
}



/******************************************************************************
	Function : Modem_Check
	Description : Check If Modem is connected
	Inputs : none
	Outputs : Returns 0 on successfull modem detect
******************************************************************************/
u08 
Modem_Check()
{
	u08 _Modem_Check, _Return=2, i, j;
	//Check If USART is enabled, if not enable it
	if(bit_is_clear(UCSRB,TXEN)) {
		Init_Modem();
	}
	//Sending AT i.e. Attention
	loop_until_bit_is_set(UCSRA, UDRE);
	UDR = 'A';
	loop_until_bit_is_set(UCSRA, UDRE);
	UDR = 'T';
	loop_until_bit_is_set(UCSRA, UDRE);	//Sending Return Character <-'
	UDR = 13;

	//Wait To Receive OK
	//Timeout & Check Rx 
	for(i = 0; i < 255; i++) {
		for(j = 0 ; j < 255; j++) {
			if(bit_is_set(UCSRA, RXC)) {
				_Return--;
				break;
			}
		}
	}
	if(_Return == 1) {
		_Modem_Check = UDR;
		if(_Modem_Check == 'O') {
			_Return--;
		}
	} else {
		return _Return;
	}
	
	_Return = 2;
	//Timeout & Check Rx
	for(i = 0; i < 255; i++) {
		for(j = 0 ; j < 255; j++) {
			if(bit_is_set(UCSRA, RXC)) {
				_Return--;
				break;
			}
		}
	}
	if(_Return == 1) {
		_Modem_Check = UDR;
		if(_Modem_Check == 'K') {
			_Return--;
		}
	} else {
		return _Return;
	}
	
	//Will return 0 on success and non zero value no error
	return (_Return);
}



/******************************************************************************
	Function : Dial_Number
	Description : Dial's A Number (Tone Dial) Using the Modem
	Inputs : 
		Prefix : Accepts Prefix To Be Dialed In BCD Format
		Pause : If non zero Will put the pause character after the Prefix
		*Number : Pointer to an array which stores the number
		Size : The size of the array i.e. the length of the number
	Outputs : none
******************************************************************************/
void
Dial_Number(u08 Prefix, u08 Pause, u08 *Number, u08 Size)
{
	u08 i;
	//Check If USART is Enabled, if not enable it
	if(bit_is_clear(UCSRB,TXEN)) {
		Init_Modem();
	}	
	//Send prefix numbers
	if(Prefix < 0x99) {
		loop_until_bit_is_set(UCSRA, UDRE);
		UDR = ((Prefix & 0xF0) >> 4) + 0x30;		
		loop_until_bit_is_set(UCSRA, UDRE);
		UDR = (Prefix & 0x0F) + 0x30;
	}
	//Send pause charcter if neccessary
	if(Pause) {
		loop_until_bit_is_set(UCSRA, UDRE);
		UDR = ',';
	}
	//Sending Number to be dialed
	for(i = 0; i < Size; i++) {
		loop_until_bit_is_set(UCSRA, UDRE);
		UDR = (0x30 + *Number);
		Number++;
	}
	loop_until_bit_is_set(UCSRA, UDRE);
	UDR = 13;
}

/******************************************************************************
	Function : Escape Sequence
	Description : This command initiates an escape sequence to return the modem
			to the on-line command mode.
	Inputs : none
	Outputs : none
******************************************************************************/
void 
Escape_Sequence(void) {
	loop_until_bit_is_set(UCSRA, UDRE);
	UDR = '+';
	loop_until_bit_is_set(UCSRA, UDRE);
	UDR = '+';
	loop_until_bit_is_set(UCSRA, UDRE);
	UDR = '+';
	loop_until_bit_is_set(UCSRA, UDRE);
	UDR = 13;
}

/******************************************************************************
	Function : Modem Info
	Description : This command reports selected information about the modem
	Inputs : I (integer) Information Type
			0  -  Modem model and speed
			1  -  ROM checksum
			2  -  Tests ROM checksum THEN reports it
			6  -  Data connection info
			7  -  Manufacturer and model info
	Outputs : Character array containing modem information
******************************************************************************/
u08 
*Modem_Info(u08 I) {
	u08 *_Info, i,j,k=0;
	loop_until_bit_is_set(UCSRA, UDRE);
	UDR = 'A';
	loop_until_bit_is_set(UCSRA, UDRE);
	UDR = 'T';
	loop_until_bit_is_set(UCSRA, UDRE);
	UDR = 'I';
	loop_until_bit_is_set(UCSRA, UDRE);
	UDR = I + 0x30;	
	for(;;) {
		BACK:
		for(i=0;i<255;i++) {
			for(j=0;j<255;j++) {
				if(bit_is_set(UCSRA, RXC)) {
					_Info[k++] = UDR;
					goto BACK;
				}
			}
		}
		break;
	}
	return (&_Info);
}

/******************************************************************************
	Function : Modem String Send
	Description : Functions send string of specified length to modem
	Inputs : 
		Str - Starting Address of string
		Length - Length of string
	Outputs : none
******************************************************************************/
void
Modem_Send_Str(u08 *Str, u08 Length) {
	//Check If USART is Enabled, if not enable it
	if(bit_is_clear(UCSRB,TXEN)) {
		Init_Modem();
	}	
	//Send String
	for(i = 0; i < Length; i++) {
		loop_until_bit_is_set(UCSRA, UDRE);
		UDR = *Str++;
	}
}
